<?php

declare(strict_types=1);

namespace Drupal\Tests\ui_patterns\Kernel;

use Drupal\Core\Link;
use Drupal\Core\Render\Markup;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Template\Attribute;
use Drupal\Core\Url;
use Drupal\KernelTests\KernelTestBase;
use Drupal\Tests\ui_patterns\Traits\TestDataTrait;

/**
 * Test prop types normalization.
 *
 * @group ui_patterns
 */
class PropTypesNormalizationTest extends KernelTestBase {

  use TestDataTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'text',
    'field',
    'node',
    'ui_patterns',
    'ui_patterns_test',
    'datetime',
    'filter',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installSchema('node', 'node_access');
    $this->installEntitySchema('node');
    $this->installEntitySchema('user');
    $this->installConfig(['system', 'filter']);
  }

  /**
   * Test prop normalization.
   *
   * @dataProvider propTypeDataProvider
   */
  public function testPropTypeNormalization(string $prop, array $tested_values) : void {
    $render_array_test = [
      "#type" => "inline_template",
      "#template" => sprintf("{{ include('ui_patterns_test:test-component', {%s: injected}) }}", $prop),
      "#context" => ["injected" => ""],
    ];
    foreach ($tested_values as $tested_value) {
      $this->assertExpectedOutput(
        [
          "rendered_value" => $tested_value["rendered_value"],
          "assert" => $tested_value["assert"] ?? "assertStringContainsString",
        ],
        array_merge($render_array_test, ["#context" => $tested_value["#context"]])
      );
    }
    // With render array.
    $render_array_test = [
      "#type" => "component",
      '#component' => 'ui_patterns_test:test-component',
      "#props" => [$prop => []],
    ];
    foreach ($tested_values as $tested_value) {
      $render_array_test["#props"][$prop] = $tested_value["#context"]["injected"];

      $this->assertExpectedOutput(
          [
            "rendered_value" => $tested_value["rendered_value"],
            "assert" => $tested_value["assert"] ?? "assertStringContainsString",
          ],
          $render_array_test
        );
    }
  }

  /**
   * Provides prop type data.
   */
  public static function propTypeDataProvider() : array {
    $returned = [];
    // Url.
    $returned["url"] = [
      "url",
      [
        ["#context" => ["injected" => "https://drupal.org"], "rendered_value" => "https://drupal.org"],
        ["#context" => ["injected" => Url::fromUri("https://drupal.org")], "rendered_value" => "https://drupal.org"],
        ["#context" => ["injected" => "/user"], "rendered_value" => "/user"],
        ["#context" => ["injected" => "internal:/user"], "rendered_value" => "/user"],
      ],
    ];
    // Links.
    $returned["links"] = [
      "links",
      [
        [
          "#context" => [
            "injected" => [
              [
                "url" => "https://drupal.org",
                "title" => "Drupal",
              ],
            ],
          ],
          "rendered_value" => '<a href="https://drupal.org">Drupal</a>',
        ],
        [
          "#context" => [
            "injected" => [
              [
                "url" => "https://drupal.org",
                "title" => 2,
              ],
            ],
          ],
          "rendered_value" => '<a href="https://drupal.org">2</a>',
        ],
        [
          "#context" => [
            "injected" => [
               [
                 "url" => Url::fromUri("https://drupal.org"),
                 "title" => Markup::create("Drupal"),
               ],
            ],
          ],
          "rendered_value" => '<a href="https://drupal.org">Drupal</a>',
        ],
        [
          "#context" => [
            "injected" => [
               [
                 "url" => "<nolink>",
                 "title" => "Drupal",
               ],
            ],
          ],
          "rendered_value" => '<div class="ui-patterns-props-url"></div>',
        ],
        [
          "#context" => [
            "injected" => [
               [
                 "url" => "<front>",
                 "title" => "<svg xmlns=\"http://www.w3.org/2000/svg\" aria-hidden=\"true\">
                           <path d=\"m8 3.293 4.712 4.712A4.5 4.5 0 0 0 8.758 15H3.5A1.5 1.5 0 0 1 2 13.5V9.293z\"></path>
                         </svg>",
               ],
            ],
          ],
          "rendered_value" => "><svg xmlns=\"http://www.w3.org/2000/svg\" aria-hidden=\"true\"><path d=\"m8 3.293 4.712 4.712A4.5 4.5 0 0 0 8.758 15H3.5A1.5 1.5 0 0 1 2 13.5V9.293z\"></path></svg></a>",
        ],
      ],
    ];
    // Enum integer.
    $returned["enum_integer"] = [
      "enum_integer",
      [
        ["#context" => ["injected" => 2], "rendered_value" => '<div class="ui-patterns-props-enum_integer">2</div>'],
        ["#context" => ["injected" => "2"], "rendered_value" => '<div class="ui-patterns-props-enum_integer">2</div>'],
        [
          "#context" => ["injected" => "BAD VALUE"],
          "rendered_value" => '<div class="ui-patterns-props-enum_integer"></div>',
        ],
        [
          "#context" => ["injected" => new \stdClass()],
          "rendered_value" => '<div class="ui-patterns-props-enum_integer"></div>',
        ],
        [
          "#context" => ["injected" => [2]],
          "rendered_value" => '<div class="ui-patterns-props-enum_integer">2</div>',
        ],
        [
          "#context" => ["injected" => ["aa" => 2]],
          "rendered_value" => '<div class="ui-patterns-props-enum_integer">2</div>',
        ],
        [
          "#context" => ["injected" => ["1" => NULL, "aa" => 2]],
          "rendered_value" => '<div class="ui-patterns-props-enum_integer">2</div>',
        ],
        [
          "#context" => ["injected" => ['#markup' => "2"]],
          "rendered_value" => '<div class="ui-patterns-props-enum_integer">2</div>',
        ],
      ],
    ];
    // enum_list_multiple.
    $returned["enum_list_multiple"] = [
      "enum_list_multiple",
      [
        [
          "#context" => ["injected" => [2]],
          "rendered_value" => '<div class="ui-patterns-props-enum_list_multiple"><span>2</span></div>',
        ],
        [
          "#context" => ["injected" => ["2"]],
          "rendered_value" => '<div class="ui-patterns-props-enum_list_multiple"><span>2</span></div>',
        ],
        [
          "#context" => ["injected" => "2"],
          "rendered_value" => '<div class="ui-patterns-props-enum_list_multiple"><span>2</span></div>',
        ],
        [
          "#context" => ["injected" => [2, 2, 2]],
          "rendered_value" =>
          '<div class="ui-patterns-props-enum_list_multiple"><span>2</span><span>2</span><span>2</span></div>',
        ],
        [
          "#context" => ["injected" => [2, "BAD", 2, 2, 444, "BAD", new Attribute(), [2]]],
          "rendered_value" =>
          '<div class="ui-patterns-props-enum_list_multiple"><span>2</span><span>2</span><span>2</span></div>',
        ],
      ],
    ];
    // enum_set.
    $returned["enum_set"] = [
      "enum_set",
      [
        [
          "#context" => ["injected" => [2]],
          "rendered_value" => '<div class="ui-patterns-props-enum_set"><span>2</span></div>',
        ],
        [
          "#context" => ["injected" => ["2"]],
          "rendered_value" => '<div class="ui-patterns-props-enum_set"><span>2</span></div>',
        ],
        [
          "#context" => ["injected" => "2"],
          "rendered_value" => '<div class="ui-patterns-props-enum_set"><span>2</span></div>',
        ],
        [
          "#context" => ["injected" => [2, 2, 2]],
          "rendered_value" =>
          '<div class="ui-patterns-props-enum_set"><span>2</span></div>',
        ],
        [
          "#context" => ["injected" => [2, "BAD", 2, 2, 444, "BAD", new Attribute(), [2]]],
          "rendered_value" =>
          '<div class="ui-patterns-props-enum_set"><span>2</span></div>',
        ],
      ],
    ];
    // Strings.
    $returned["string"] = [
      "string",
      [
        ["#context" => ["injected" => "my string"], "rendered_value" => "my string"],
        [
          "#context" => ["injected" => ['#markup' => "my string2"]],
          "rendered_value" => "my string2",
        ],
        [
          "#context" => ["injected" => Markup::create("my string3")],
          "rendered_value" => "my string3",
        ],
        [
          "#context" => ["injected" => Url::fromUri("https://drupal.org")],
          "rendered_value" => "https://drupal.org",
        ],
        [
          "#context" => [
            "injected" =>
            Link::fromTextAndUrl(Markup::create("test"), Url::fromUri("https://drupal.org")),
          ],
          "rendered_value" =>
          '<div class="ui-patterns-props-string">&lt;a href=&quot;https://drupal.org&quot;&gt;test&lt;/a&gt;</div>',
        ],
        ["#context" => ["injected" => ""], "rendered_value" => ""],
        ["#context" => ["injected" => NULL], "rendered_value" => ""],
        ["#context" => ["injected" => 2], "rendered_value" => "2"],
      ],
    ];
    // Variants.
    $returned["variant"] = [
      "variant",
      [
        ["#context" => ["injected" => "default"], "rendered_value" => "ui-patterns-test-component-variant-default"],
        ["#context" => ["injected" => ""], "rendered_value" => "ui-patterns-test-component-variant-"],
        ["#context" => ["injected" => "other"], "rendered_value" => "ui-patterns-test-component-variant-other"],
        ["#context" => ["injected" => "BAD"], "rendered_value" => "ui-patterns-test-component-variant-"],
        ["#context" => ["injected" => 2], "rendered_value" => "ui-patterns-test-component-variant-"],
        ["#context" => ["injected" => []], "rendered_value" => "ui-patterns-test-component-variant-"],
        ["#context" => ["injected" => new \stdClass()], "rendered_value" => "ui-patterns-test-component-variant-"],
      ],
    ];
    // Test slot values.
    $tested_slot_values = [
      Markup::create("my slot"),
      new TranslatableMarkup("my slot"),
      ["my slot"],
      "my slot",
      [Markup::create("my slot")],
      ["uu" => Markup::create("my slot")],
      ["#markup" => "my slot"],
      ["a" => ["#markup" => "my "], "b" => ["#markup" => "slot"]],
      ["a" => ["#markup" => "my "], "b" => ["slot"]],
      ["b" => ["#weight" => 2, "#markup" => "slot"], "a" => ["#weight" => 1, "#markup" => "my "]],
      ["aa" => "my slot"],
      t("my slot"),
      ["#type" => "inline_template", "#template" => "my slot"],
    ];
    $returned["slot"] = [
      "slot",
      [],
    ];
    foreach ($tested_slot_values as $tested_value) {
      $returned["slot"][1][] = ["#context" => ["injected" => $tested_value], "rendered_value" => "my slot"];
    }
    return $returned;
  }

  /**
   * Test slot normalization.
   */
  public function testNestedComponentWithForm() : void {
    // Test nested component with form.
    $render_array_tests = [
      [
        "#type" => "inline_template",
        "#template" => "
        {% set comp_form = include('ui_patterns_test:test-form-component', {}) %}
        {{ include('ui_patterns_test:test-component', {slot: comp_form}) }}",
        "#context" => [],
      ],
      [
        "#type" => "component",
        '#component' => 'ui_patterns_test:test-component',
        "#slots" => [
          "slot" => [
            "#type" => "component",
            '#component' => 'ui_patterns_test:test-form-component',
          ],
        ],
      ],
    ];
    foreach ($render_array_tests as $render_array_test) {
      $this->assertExpectedOutput(
        [
          "rendered_value" => "<input ",
          "assert" => "assertStringContainsString",
        ],
        $render_array_test
      );
      $this->assertExpectedOutput(
        [
          "rendered_value" => "<form ",
          "assert" => "assertStringContainsString",
        ],
        $render_array_test
      );
    }
  }

}
